package net.hostsharing.hsadminng.rbac.rbacdef;

import net.hostsharing.hsadminng.hs.office.debitor.HsOfficeDebitorEntity;
import net.hostsharing.hsadminng.hs.office.partner.HsOfficePartnerEntity;
import net.hostsharing.hsadminng.hs.office.relationship.HsOfficeRelationshipEntity;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.nio.file.StandardOpenOption;
import java.time.LocalDateTime;


public class RbacViewPostgresGenerator {


    private final RbacView rbacDef;
    private final String liqibaseTagPrefix;
    private final StringWriter plPgSql = new StringWriter();

    public RbacViewPostgresGenerator(final RbacView forRbacDef) {
        rbacDef = forRbacDef;
        liqibaseTagPrefix = rbacDef.getRootEntityAlias().entityClass().getSimpleName();
        plPgSql.writeLn("""
                --liquibase formatted sql
                -- generated at: %{timestamp}
                """
                .replace("%{timestamp}", LocalDateTime.now().toString()));

        new RolesGrantsAndPermissionsGenerator(rbacDef, liqibaseTagPrefix).generateTo(plPgSql);
    }

    @Override
    public String toString() {
    return plPgSql.toString();
}

    private static void generatePostgres(final RbacView rbac) throws IOException {
        final Path outputPath = Paths.get("doc", rbac.getRootEntityAlias().simpleName() + ".sql");
        Files.writeString(
                outputPath,
                new RbacViewPostgresGenerator(rbac).toString(),
                StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);

        System.out.println(outputPath.toAbsolutePath());
    }

    public static void main(String[] args) throws IOException {
        generatePostgres(HsOfficeRelationshipEntity.rbac());
        generatePostgres(HsOfficePartnerEntity.rbac());
        generatePostgres(HsOfficeDebitorEntity.rbac());
    }
}
