package net.hostsharing.hsadminng.rbac.rbacdef;

import org.apache.commons.lang3.StringUtils;

import static java.util.Arrays.stream;
import static java.util.stream.Collectors.joining;

public class StringWriter {

    private final StringBuilder string = new StringBuilder();
    private int indentLevel = 0;

    void writeLn(final String text) {
        string.append( indented(text));
        writeLn();
    }

    void writeLn() {
        string.append( "\n");
    }

    private String indented(final String text) {
        if ( indentLevel == 0) {
            return text;
        }
        final var indentation = StringUtils.repeat("    ", indentLevel);
        final var indented = stream(text.split("\n"))
                .map(line -> line.trim().isBlank() ? "" : indentation + line)
                .collect(joining("\n"));
        return indented;
    }

    void indent() {
        ++indentLevel;
    }

    void unindent() {
        --indentLevel;
    }

    void indented(final Runnable indented) {
        indent();
        indented.run();
        unindent();
    }

    boolean chopTail(final String tail) {
        if (string.toString().endsWith(tail)) {
            string.setLength(string.length() - tail.length());
            return true;
        }
        return false;
    }

    void chopEmptyLines() {
        while (string.toString().endsWith("\n\n")) {
            string.setLength(string.length() - 1);
        };
    }

    void emptyLine() {
        if (!string.toString().endsWith("\n\n")) {
            writeLn();
        }
    }

    @Override
    public String toString() {
        return string.toString();
    }
}
