package net.hostsharing.hsadminng.test.cust;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import net.hostsharing.hsadminng.rbac.rbacdef.RbacView;
import net.hostsharing.hsadminng.rbac.rbacobject.RbacObject;

import jakarta.persistence.*;
import java.util.UUID;

import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.GLOBAL;
import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.Permission.ALL;
import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.Permission.VIEW;
import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.RbacUserReference.UserRole.CREATOR;
import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.Role.*;
import static net.hostsharing.hsadminng.rbac.rbacdef.RbacView.rbacViewFor;

@Entity
@Table(name = "test_customer_rv")
@Getter
@Setter
@NoArgsConstructor
@AllArgsConstructor
public class TestCustomerEntity implements RbacObject {

    @Id
    @GeneratedValue
    private UUID uuid;

    private String prefix;
    private int reference;

    @Column(name = "adminusername")
    private String adminUserName;



    public static RbacView rbac() {
        return rbacViewFor("customer", TestCustomerEntity.class)
                .withIdentityView(RbacView.SQL.query("target.prefix"))
                .withUpdatableColumns("reference", "prefix", "adminUserName")
                .createRole(OWNER, (with) -> {
                    with.owningUser(CREATOR);
                    with.incomingSuperRole(GLOBAL, ADMIN);
                    with.permission(ALL);
                })
                .createSubRole(ADMIN, (with) -> {
                    with.permission(RbacView.Permission.custom("add-package"));
                })
                .createSubRole(TENANT, (with) -> {
                    with.permission(VIEW);
                });
    }
}
